﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Infragistics.Win;
using Infragistics.Win.UltraWinGrid;
using System.Drawing;

namespace WNG_DragAndDropRowIndicator
{
    [System.Flags]
    internal enum GridRowDropLinePositionEnum
    {
        None = 0,
        AboveItem = 1,
        BelowItem = 2,
    }

    public class DragDropRowDrawFilter: IUIElementDrawFilter
    {
        #region Fields and Properties
        private UltraGridRow dropHighllightRow = null;
        internal GridRowDropLinePositionEnum dropLinePosition { get; set; }        

        public Color DropLineColor { get; set; }
        public int DropLineWidth { get; set; }

        public UltraGridRow DropHighlightRow
        {
            get { return dropHighllightRow; }
            set
            {
                if (dropHighllightRow.Equals(value))
                {
                    this.dropHighllightRow = value;
                    PositionChanged();
                }
            }
        }
        #endregion

        #region Events

        public event System.EventHandler Invalidate;

        #endregion

        #region Methods

        private void PositionChanged()
        {
            // if nobody is listening then just return
            if (Invalidate == null)
                return;

            EventArgs e = EventArgs.Empty;

            this.Invalidate(this, e);
        }

        internal void ClearDropHighlight()
        {
            SetDropHighlight(null, GridRowDropLinePositionEnum.None);
        }

        internal void SetDropHighlight(UltraGridRow _row, GridRowDropLinePositionEnum _dropLinePosition)
        {
            bool _isPostitionChanged = false;

            if (dropHighllightRow == null)
            {
                _isPostitionChanged = !(_row == null);
            }
            else
            {
                //Check to see if the items are equal and if 
                //the indicator position are equal
                if (dropHighllightRow.Equals(_row) && (dropLinePosition == _dropLinePosition))
                {
                    //They are both equal. Nothing has changed. 
                    _isPostitionChanged = false;
                }
                else
                {
                    //One or both have changed
                    _isPostitionChanged = true;
                }
            }

            //Set both properties without calling PositionChanged
            dropHighllightRow = _row;
            dropLinePosition = _dropLinePosition;

            //Check to see if the PositionChanged
            if (_isPostitionChanged)
            {
                //Position did change.
                PositionChanged();
            }
        }

        internal void SetDropHighlight(UltraGridRow _row, Point _pointInGrid)
        {
            // The distance from the edge of the item used to determine whether to 
            // drop Above, Below
            int _distanceFromEdge = _row.GetUIElement().Rect.Height / 2;

            //the new indicator position
            GridRowDropLinePositionEnum _newDropLinePostion;

            //determind which part of the row the point is in
            if (_pointInGrid.Y <= (_row.GetUIElement().Rect.Top + _distanceFromEdge))
            {
                //point is in the top of the row
                _newDropLinePostion = GridRowDropLinePositionEnum.AboveItem;
            }
            else
            {
                //point is in the bottom of the row
                _newDropLinePostion = GridRowDropLinePositionEnum.BelowItem;
            }

            // Now that we have the new DropLinePosition, call the real proc to get things rolling
            SetDropHighlight(_row, _newDropLinePostion);
        }
        
        #endregion

        #region IUIElementDrawFilter Members

        public bool DrawElement(DrawPhase drawPhase, ref UIElementDrawParams drawParams)
        {
            switch (drawPhase)
            {
                case DrawPhase.AfterDrawElement:
                    {
                        // We're in AfterDrawElement. So the only states we are concerned with are Below and Above
                        // Get the elemnt as an RowUIElement if it is one.
                        RowUIElement element = drawParams.Element as RowUIElement;
                        if (element != null && dropLinePosition != GridRowDropLinePositionEnum.None)
                        {
                            UltraGridRow rowToDrawOn = DropHighlightRow;
                            GridRowDropLinePositionEnum _dropLinePosition = this.dropLinePosition;

                            //Check to see if this is not the last row in the grid and if the indicator position is below
                            if (rowToDrawOn.Index != rowToDrawOn.ParentCollection.Count - 1 && _dropLinePosition == GridRowDropLinePositionEnum.BelowItem)
                            {
                                //If we need to draw the indicator below a row, we will have to change the rowToDrawOn to the row below
                                //This is because the row below's top border will overwrite the border below the UIElement we are Dragging over
                                //To account for this we will set the rowToDraw on to the next row below and the indicator position to be above                                
                                //We will not want to change the rowToDrawOn if this is the last row because we CAN draw the last row's border
                                //without it being overwritten by any rows below
                                rowToDrawOn = rowToDrawOn.ParentCollection[rowToDrawOn.Index + 1];
                                _dropLinePosition = GridRowDropLinePositionEnum.AboveItem;
                            }

                            //if we have not changed the rowToDrawOn, this logic will pass for the row being dragged over
                            //if we have changed the rowToDrawOn, this logic will pass on the next call to this function for the row below
                            if (rowToDrawOn == element.Row)
                            {
                                // Declare a pen to use for drawing the indicator
                                using (Pen p = new Pen(DropLineColor, DropLineWidth))
                                {
                                    Graphics g = drawParams.Graphics;

                                    //the left edge of the indicator
                                    int leftEdge = element.Rect.Left;

                                    //the right edge of the indicator
                                    int rightEdge = element.Rect.Right;

                                    int lineVPosition;

                                    if (_dropLinePosition == GridRowDropLinePositionEnum.BelowItem)
                                    {
                                        //Draw the indicator below
                                        lineVPosition = element.Rect.Bottom;
                                        g.DrawLine(p, leftEdge, lineVPosition, rightEdge, lineVPosition);
                                    }
                                    if (_dropLinePosition == GridRowDropLinePositionEnum.AboveItem)
                                    {
                                        //Draw the indicator above
                                        lineVPosition = element.Rect.Top;
                                        g.DrawLine(p, leftEdge, lineVPosition, rightEdge, lineVPosition);
                                    }

                                }
                            }
                        }
                    }
                    break;
            }
            return false;
        }

        public DrawPhase GetPhasesToFilter(ref UIElementDrawParams drawParams)
        {
            return DrawPhase.AfterDrawElement;
        }

        #endregion
    }
}
